import warnings
warnings.filterwarnings('ignore')
from . _traceprinter import TracerPrinterWrapper as trace
import json

class JsonOpsWrapper:
    '''This class interacts with mongodb and performs mongodb operations.'''
        
    '''This class interacts with sql and performs sql operations.'''

    def write_df_to_json(df, jsonfilename, tracing = False):
        try:
            trace.print(f'''write df to json: 
                        \t, jsonfilename: {jsonfilename}''', tracing)
            
            b = False
            tojson = df.to_json(orient = 'records', date_format = 'iso')
            list_json = json.loads(tojson)

            with open(jsonfilename, "w") as outfile: #"logs\custom_logs.json"
                json.dump(list_json, outfile)
                trace.print(f'write df to json executed,', tracing)
                b = True
        except Exception as ex:
            trace.print(f'write df to json failed,', tracing)
            raise Exception(ex)
        return b

    def read_json(jsonfilename, tracing = False):
        try:
            trace.print(f'''read json file: 
                        \t, jsonfilename: {jsonfilename}''', tracing)
            
            with open(jsonfilename) as json_file:
                data = json.load(json_file)
                json_data = data
                trace.print(f'read json file executed,', tracing)
            return json_data
        except Exception as ex:
            trace.print(f'read json file failed,', tracing)
            raise Exception(ex)


import pandas as pd
import warnings
warnings.filterwarnings('ignore')
from pymongo import MongoClient
from . _traceprinter import TracerPrinterWrapper as trace
import datetime
from dateutil import parser

class MongodbOpsWrapper:
    '''This class interacts with mongodb and performs mongodb operations.'''
        
    '''This class interacts with sql and performs sql operations.'''
    def __init__(self, cs_mongodb_url):
        #url = "mongodb://localhost:27017"
        mongoclient = MongoClient(cs_mongodb_url)
        #print(f'oms traces, mongodb connected,')
        self.__client = mongoclient
    
    def __del__(self):
        #print("oms traces, mongodb obj being automatically destroyed,")
        pass


    def check_connection(self, tracing = False):
        """It executes the select sql statement provided as sql query.
        
        Parameters:
            query (str): sql query to select data

        Returns: 
            df (dataframe): returns dataframe
        refer: https://www.mongodb.com/docs/manual/reference/method/db.collection.find/
        """
        trace.print(f'check mongodb connection,', tracing)
        status = 'mongodb connection failed.'
        try:
            _ = self.__client.list_databases()

            status = 'mongodb connection succeeded.'
            trace.print(f'check mongodb connection executed,', tracing)
        except Exception as ex:
            status = 'mongodb connection failed.'
            trace.print(f'mongodb connection failed,', tracing)
            raise Exception(ex)
        return status
    
    def querycollection(self, db, collection, query, projection, tracing = False):
        """It executes the select sql statement provided as sql query.
        
        Parameters:
            query (str): sql query to select data

        Returns: 
            df (dataframe): returns dataframe
        refer: https://www.mongodb.com/docs/manual/reference/method/db.collection.find/
        """
        try:
            trace.print(f'''query collection, db: {db}
                        \t, collection: {collection}
                        \t, query: {query}
                        \t, projection: {projection}''', tracing)

            db = self.__client[db]
            if query is None:
                q = {}
            else:
                q = query
            
            if projection is None:
                collection = db.get_collection(collection).find(q)
            else:
                collection = db.get_collection(collection).find(q, projection)

            trace.print(f'query collection executed,', tracing)
            return collection
        except Exception as ex:
            trace.print(f'query collection failed,', tracing)
            raise Exception(ex)
    
    def querycollection_into_df(self, db, collection, query, projection, tracing = False):
        """It executes the select sql statement provided as sql query.
        
        Parameters:
            query (str): sql query to select data

        Returns: 
            df (dataframe): returns dataframe
        """
        try:
            trace.print(f'''query collection into df,''', tracing)
            results = self.querycollection(db, collection, query, projection, tracing)

            df = pd.DataFrame(list(results))
            trace.print(f'query collection into df executed,', tracing)

            return df
        except Exception as ex:
            trace.print(f'query collection into df failed,', tracing)
            raise Exception(ex)
 
    def insert_df_into_collection(self, df, db, collection, tracing = False):
        """It executes the select sql statement provided as sql query.
        
        Parameters:
            query (str): sql query to select data

        Returns: 
            df (dataframe): returns dataframe
        """

        b = False
        trace.print(f'insert df into db: {db}, collection: {collection}', tracing)
        try:
            db = self.__client[db]
            
            # df.reset_index(inplace = True) #if you need a default index
            data_dict = df.to_dict("records")
            collection = db[collection]

            collection.insert_many(data_dict)
            trace.print(f'df inserted into {db}.{collection} executed successfully,', tracing)
            b = True
            return b
        except Exception as ex:
            trace.print(f'insert df failed,', tracing)
            raise Exception(ex)

    def join_collections_into_df(self, list_coll_details, list_how, list_on, tracing = False):
        """It executes the select sql statement provided as sql query.
        
        Parameters:
            query (str): sql query to select data

        Returns: 
            df (dataframe): returns dataframe
        """
        try:
            trace.print(f'joining collections...', tracing)

            list_df = []
            for d in list_coll_details:
                df = self.querycollection_into_df(d.get('db'),
                                                d.get('coll'),
                                                None,
                                                d.get('project'),
                                                tracing
                                                )
                list_df.append(df)

            no_of_joins = len(list_df) - 1
            
            if no_of_joins > 0:
                dfj = list_df[0].merge(list_df[1]
                                    , how = list_how[0]
                                    , left_on = list_on[0].get('left_on')
                                    , right_on = list_on[0].get('right_on'))
                
                if no_of_joins > 1:
                    for i in range(1, no_of_joins):
                        dfj = dfj.merge(list_df[i+1]
                                        , how = list_how[i]
                                        , left_on = list_on[i].get('left_on')
                                        , right_on = list_on[i].get('right_on')
                                        )
            
            trace.print(f'joining collections executed,', tracing)
            return dfj
        except Exception as ex:
            trace.print(f'joining collections failed,', tracing)
            raise Exception(ex)

    def generate_sample_data(self, db, tracing = False):
        try:
            # ### prepare some sample data first
            registration_date = '2023-07-11T00:00:00.000Z'
            registration_date = parser.parse(registration_date)
            dt = datetime.datetime.now(tz = datetime.timezone.utc)

            data = [{"name" : "om", "course" : "SQL", "fees" : 265, "currency" : "usd", "add" : {"state": "up", 'country':'india'}, "reg_date": datetime.datetime(2009, 11, 12, 11, 14) },
                    {"name" : "om shakya", "course" : "SQL", "fees" : 270, "currency" : "usd", "add" : {"state": "ny", 'country':'america'}, "reg_date" : registration_date },
                    {"name" : "om prakash shakya", "course" : "python", "fees" : 250, "currency" : "inr", "add" : {"state": "delhi", 'country':'india'}, "reg_date" : parser.parse('2023-07-11 10:12:36')},
                    {"name" : "kanika", "course" : "python", "fees" : 250, "currency" : "inr", "add" : {"state": "delhi", 'country':'india'}, "reg_date" : dt},
                    {"name" : "alex", "course" : "SQL", "fees" : 275, "currency" : "inr", "add" : {"state": "delhi", 'country':'india'}, "reg_date" : dt},
                    {"name" : "James", "course" : "pyspark", "fees" : 285, "currency" : "usd", "add" : {"state": "ny", 'country':'america'}, "reg_date" : dt},
                    {"name" : "Joseph", "course" : "SQL", "fees" : 290, "currency" : "usd", "add" : {"state": "al", 'country':'america'}, "reg_date" : dt},
                    {"name" : "Elizabeth", "course" : "python", "fees" : 240, "currency" : "usd", "add" : {"state": "fs", 'country':'america'}, "reg_date" : dt},
                    {"name" : "Noah", "course" : "SQL", "fees" : 278, "currency" : "inr", "sg" : {"state": "delhi", 'country':'singapore'}, "reg_date" : dt},
                    ]

            # ### create db & collection and insert sample data
            db = self.__client[db]
            collection = 'registrations'
            collection = db.get_collection(collection)
            collection.insert_many(data)

            # ### one more sample data
            data_curr = [{"currency" : "sd", "description" : "singapore dollar"},
                    {"currency" : "usd", "description" : "united states dollar"},
                    {"currency" : "inr", "description" : "indian rupee"},
                    {"currency" : "gbp", "description" : "great britain pound"}
                    ]
            collection = db.get_collection('currencies')
            collection.insert_many(data_curr)

            # ### one more sample data
            data_fees = [{"amount" : 250, "in words" : "two fifty"},
                         {"amount" : 260, "in words" : "two sixty"},
                         {"amount" : 270, "in words" : "two seventy"},
                         {"amount" : 275, "in words" : "two seventy five"}
                         ]
            db = self.__client['test_db']
            collection = db.get_collection('fees')
            collection.insert_many(data_fees)

            list_cols = ['registrations', 'currencies', 'fees']

            trace.print('sample data generated successfully,', tracing)
            return list_cols
        
        except Exception as ex:
            trace.print(f'sample data generation failed,', tracing)
            raise Exception(ex)

    def mongo_ops_1():
        print('mongo_ops_1')




# ### these both syntax works fine
#from omshakya._sharedcode import _mongodbops as __sqlo
from .. _bin_cc_or_sc import _mongodbops as __mongo
from .. _bin_cc_or_sc import __version__ as __v
__version__ = __v.__VERSION__


def check_connection(cs_mongodb_url, tracing = False):
    """
    Check the connection to the mongodb database.
    
    Parameters:
        cs_mongodb_url (str): the mongodb url to connect with, ex. "mongodb://localhost:27017"
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        str: The staus of connection.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __mongo.MongodbOpsWrapper(cs_mongodb_url)
    return __obj.check_connection(tracing)

def generate_sample_data(cs_mongodb_url, db, tracing = False):
    """
    Generate the sample data in the mongodb database.
    
    Parameters:
        cs_mongodb_url (str): the mongodb url to connect with, ex. "mongodb://localhost:27017"
        db (str): the name of the database, it will be created if does not exists
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        List(str): The list of collection names created with sample data.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __mongo.MongodbOpsWrapper(cs_mongodb_url)
    return __obj.generate_sample_data(db, tracing)

def insert_df_into_collection(cs_mongodb_url, df, db, collection, tracing = False):
    """
    Insert pandas datafrom into mongodb database collection.
    
    Parameters:
        cs_mongodb_url (str): the mongodb url to connect with, ex. "mongodb://localhost:27017"
        df (pandas dataframe): pandas dataframe
        db (str): the name of the database, it will be created if does not exists
        collection (str): the name of the collection, it will be created if does not exists
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        True/False: 'True' if datafrae inserted into collection else 'False'
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __mongo.MongodbOpsWrapper(cs_mongodb_url)
    return __obj.insert_df_into_collection(df, db, collection, tracing)

def join_collections_into_df(cs_mongodb_url, list_coll_details, list_how, list_on, tracing = False):
    """
    Joins multiple collections in mongodb database. This is like 'sql join' or 'mongodb lookup' functionality.
    
    Parameters:
        cs_mongodb_url (str): the mongodb url to connect with, ex. "mongodb://localhost:27017"
        list_coll_details (List(Dict)): list of dictionaries to provide collections details
        list_how (List(str)): how to join the collections, 'left', 'right', 'inner' etc.
        list_on (List(str)): on which columns the join will be performed
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        df (pandas dataframe): the output pandas dataframe after joining collections.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __mongo.MongodbOpsWrapper(cs_mongodb_url)
    return __obj.join_collections_into_df(list_coll_details, list_how, list_on, tracing)

def querycollection(cs_mongodb_url, db, collection, query, projection, tracing = False):
    """
    Query the mongodb database collection.
    
    Parameters:
        cs_mongodb_url (str): the mongodb url to connect with, ex. "mongodb://localhost:27017"
        db (str): the name of the database, it will be created if does not exists
        collection (str): the name of the collection, it will be created if does not exists
        query (dict): the mongodb query
        projection (dict): the mongodb projection
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        cursor: the output of mongodb query as cursor.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __mongo.MongodbOpsWrapper(cs_mongodb_url)
    return __obj.querycollection(db, collection, query, projection, tracing)

def querycollection_into_df(cs_mongodb_url, db, collection, query, projection, tracing = False):
    """
    Query the mongodb database collection into the pandas dataframe.
    
    Parameters:
        cs_mongodb_url (str): the mongodb url to connect with, ex. "mongodb://localhost:27017"
        db (str): the name of the database, it will be created if does not exists
        collection (str): the name of the collection, it will be created if does not exists
        query (dict): the mongodb query
        projection (dict): the mongodb projection
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        df (pandas dataframe): the output of mongodb query into pandas dataframe.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __mongo.MongodbOpsWrapper(cs_mongodb_url)
    return __obj.querycollection_into_df(db, collection, query, projection, tracing)




import warnings
warnings.filterwarnings('ignore')
import pandas as pd

class TracerPrinterWrapper:
    '''prints any object which you want in between execution as tracing.'''

    def print(toprint, display):
        if display:
            isdf = isinstance(toprint, pd.DataFrame)
            if isdf:
                print('oms traces, df:')
                print(toprint)  
            else:
                print(f'oms traces, {toprint}')  



from ._bin_cc_or_sc import __version__ as v
__version__ = v.__VERSION__

def evaluate_package_requirements():
    """
    Evaluate the current project environment to check the required packages.

    As of now, the following packages are required to use this package 'omshakya':
    1. pandas
    2. sqlalchemy
    3. pymongo

    It will check and report the status on screen/console.
    """
    print('evaluating package requirements...')
    import pkg_resources
    packages = ['pandas', 'sqlalchemy', 'pymongo']
    print('===='*16)
    
    installed = ''
    notinstalled = ''
    toinstall = ''
    for package in packages:
        try:	
            dist = pkg_resources.get_distribution(package)
            installed = installed + f'"{dist}" is installed\n'
        except pkg_resources.DistributionNotFound:
            notinstalled = notinstalled + f'"{package}" is not installed\n'
            toinstall = toinstall + f'pip install {package}\n'
    
    if not(notinstalled == ''):
        print(f'sad!, this package "omshakya" depends on these packages')
        print(f'{notinstalled}')
        print(f'please install,')
        print(f'{toinstall}')
    else:
        print(f'congratulation!, all the required packages are installed,')
        print(f'{installed}')
    print('===='*16)



# ### these both syntax works fine
#from omshakya._sharedcode import _sqloperations as __sqlo
from .._bin_cc_or_sc import _sqlops as __sqlo
from .. _bin_cc_or_sc import __version__ as __v
__version__ = __v.__VERSION__


def check_connection(cs_odbc_url, tracing = False):
    """
    Check the connection to the sql server.
    
    Parameters:
        cs_odbc_url (str): the microsoft sql server odbc connection string also known as odbc url
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        str: The staus of connection.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __sqlo.SqlOpsWrapper(cs_odbc_url)
    return __obj.check_connection(tracing)

def select_from_sql(cs_odbc_url, script, tracing = False):
    """
    Check the connection to the sql server.
    
    Parameters:
        cs_odbc_url (str): the microsoft sql server odbc connection string also known as odbc url
        script (str): the microsoft sql select statement, can be any kind of select statement ex. select with 
            subqueries, select with CTE, select from views, select from function.
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        str: The staus of connection.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __sqlo.SqlOpsWrapper(cs_odbc_url)
    return __obj.select_by_script(script, tracing)

def execute_update_sql_script(cs_odbc_url, script, tracing = False):
    """
    Execute the microsoft sql update statement.
    
    Parameters:
        cs_odbc_url (str): the microsoft sql server odbc connection string also known as odbc url
        script (str): the microsoft sql update statement
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        count (int): The nuber of records affected by update statement.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __sqlo.SqlOpsWrapper(cs_odbc_url)
    return __obj.execute_updatescript(script, tracing)

def drop_table(cs_odbc_url, schemaname, tblname, tracing = False):
    """
    Drop the table from microsoft sql databse.
    
    Parameters:
        cs_odbc_url (str): the microsoft sql server odbc connection string also known as odbc url
        schemaname (str): the microsoft sql database schema name
        tblname (str): the microsoft sql database table name
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        True/False: 'True' if table was deleted else 'False'.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __sqlo.SqlOpsWrapper(cs_odbc_url)
    return __obj.drop_table(schemaname, tblname, tracing)

def execute_script(cs_odbc_url, script, tracing = False):
    """
    Execute the microsoft sql script (T-SQL). Can execute any (UPDATE, DELETE, TRUNCATE), DDL ,DCL, Stored procedures without parameters,
    SQL queries/commands except SELECT & DELETE
    
    Parameters:
        cs_odbc_url (str): the microsoft sql server odbc connection string also known as odbc url
        script (str): the microsoft sql script, Can be any (UPDATE, DELETE, TRUNCATE), DDL ,DCL, 
            Stored procedures without parameters. T-SQL queries/commands except SELECT & DELETE

        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        int: scalar integer result or number of rows affected. -2 means it has failed.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __sqlo.SqlOpsWrapper(cs_odbc_url)
    return __obj.execute_script(script, tracing)

def load_df_intotable(cs_odbc_url, df, schema_name, table_name, replace_append, tracing = False):
    """
    Load pandas dataframe into microsoft sql databse table.
    
    Parameters:
        cs_odbc_url (str): the microsoft sql server odbc connection string also known as odbc url
        df (pandas dataframe): the pandas dataframe to load into sql database table
        schema_name (str): the schema name ex. 'dbo'
        table_name (str): the table name ex. 'employee_table'
        replace_append (str): whether to append or replace, 'a' for append & 'r' for replace
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        True/False: 'True' if table was deleted else 'False'.
    Example:
        See the documentation at 'https://www.linkedin.com/company/omshakya-a-python-package/?viewAsMember=true'
    """
    __obj = __sqlo.SqlOpsWrapper(cs_odbc_url)
    return __obj.load_df_intotable(df, schema_name, table_name, replace_append, tracing)



# ### these both syntax works fine
#from omshakya._sharedcode import _sqloperations as __sqlo
from .. _bin_cc_or_sc import _jsonops as __jsono

from .. _bin_cc_or_sc import __version__ as __v
__version__ = __v.__VERSION__

"""
Utility functions related to json.
"""

def write_df_to_jsonfile(df, jsonfilename, tracing = False):
    """
    Write the pandas dataframe into a .json file.
    
    Parameters:
        df (pandas dataframe): pandas dataframe
        jsonfilename (str): the name/path of the json file to write
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        True/False: 'True' if dataframe written to json file else 'False'
    """
    return __jsono.JsonOpsWrapper.write_df_to_json(df, jsonfilename, tracing)

def read_jsonfile(jsonfilename, tracing = False):
    """
    Read the .json file.
    
    Parameters:
        jsonfilename (str): the name/path of the json file to read
        tracing (boolean): True/False, whether to print the progress of the function code execution

    Returns: 
        json data: The json data of the file
    """
    return __jsono.JsonOpsWrapper.read_json(jsonfilename, tracing)



import pandas as pd
import sqlalchemy as sa
from sqlalchemy.sql import text
import warnings
warnings.filterwarnings('ignore')
from . _traceprinter import TracerPrinterWrapper as trace


class SqlOpsWrapper:
    '''This class interacts with sql and performs sql operations.'''
    def __init__(self, cs_odbc_url):
        conn_url = sa.engine.URL.create("mssql+pyodbc", query = dict(odbc_connect = cs_odbc_url))
        engine = sa.create_engine(conn_url, fast_executemany = True)
        self.__engine = engine
    
    def __del__(self):
        #print("being automatically destroyed,")
        pass

    def check_connection(self, tracing = False):
        """It executes the select sql statement provided as sql query.
        
        Parameters:
            query (str): sql query to select data

        Returns: 
            df (dataframe): returns dataframe
        """
        trace.print(f'check sql connection,', tracing)
        status = 'sql connection failed.'
        try:
            q = 'SELECT TOP 1 object_id FROM sys.tables;'
            _ = self.select_by_script(q, tracing)
            status = 'sql connection succeeded.'
            trace.print(f'check sql connection executed,', tracing)
        except Exception as ex:
            status = 'sql connection failed.'
            trace.print(f'check sql connectionfailed,', tracing)
            raise Exception(ex)
        return status

    def select_by_script(self, query, tracing = False):
        """It executes the select sql statement provided as sql query.
        
        Parameters:
            query (str): sql query to select data

        Returns: 
            df (dataframe): returns dataframe
        """
        try:
            trace.print(f'''select by script, ''', tracing)
            trace.print(text(query), tracing)

            with self.__engine.connect() as connection:
                df = pd.read_sql_query(text(query), connection)

            trace.print(f'select by script executed successfully,', tracing)
            return df
        except Exception as ex:
            trace.print(f'select by script failed,', tracing)
            raise Exception(ex)
    
    def execute_updatescript(self, script, tracing = False):
        """It executes the sql script provided.
        
        Parameters:
            script (str): sql script
            is_result (boolean): do you need the scalar result of connection.execute(script)?

        Returns: 
            counts (scalar or number of records) : bydeafault counts = 0 which means script executed succesfully
            , counts = -1 which means some exception occured
            , otherwise  scalar result
        """
        counts = 0
        trace.print(f'''execute update script, ''', tracing)
        trace.print(text(script), tracing)
        try:
            with self.__engine.connect().execution_options(autocommit = True) as connection:
                results = connection.execute(text(script))
                
                try:
                    counts = results.rowcount
                    trace.print(f'results.rowcount: {counts}', tracing)
                except Exception as ex:
                    pass

            trace.print(f'counts: {counts}', tracing)
            trace.print(f'execute update script executed successfully,', tracing)
            return counts
        except Exception as ex:
            trace.print(f'execute update script failed,', tracing)
            raise Exception(ex)

        #######################################
        
    def drop_table(self, schemaname, tblname, tracing = False):
        """Prepares the sql statement to drop a table and drops the table.
        
        Parameters:
            schemaname (str): schema name
            tblname (str): table name

        Returns:
            b (boolean): returns status whether table droped or not
        """
        try:
            trace.print(f'''drop table, ''', tracing)
            
            sql = f"""IF  EXISTS (
                SELECT * FROM sys.objects 
                WHERE object_id = OBJECT_ID(N'{schemaname}.{tblname}')
                AND type in (N'U'))
                DROP TABLE {schemaname}.{tblname}"""
            trace.print(text(sql), tracing)
            
            b = False
            with self.__engine.connect().execution_options(autocommit = True) as connection:
                try:
                    connection.execute(text(sql))
                    b = True
                except Exception as ex:
                    raise Exception(ex)
                
            trace.print(f'drop table executed successfully,', tracing)
            return b
        except Exception as ex:
            trace.print(f'drop table failed,', tracing)
            raise Exception(ex)

    def execute_script(self, script, tracing = False):
        """It executes the sql script provided.
        
        Parameters:
            script (str): sql script
            is_result (boolean): do you need the scalar result of connection.execute(script)?

        Returns: 
            counts (scalar or number of records) : bydeafault counts = 0 which means script executed succesfully
            , counts = -2 which means some exception occured
            , otherwise  scalar result
        """
        trace.print(f'''execute script, ''', tracing)
        trace.print(text(script), tracing)
        counts = 0
        try:
            with self.__engine.connect().execution_options(autocommit = True) as connection:
                results = connection.execute(text(script))
            
            try:
                counts = results.rowcount
                trace.print(f'results.rowcount: {counts}', tracing)
            except Exception as ex:
                counts = -2
                pass

            trace.print(f'counts: {counts}', tracing) 
            trace.print(f'execute script executed successfully,', tracing) 
            return counts  
        except Exception as ex:
            trace.print(f'execute script executed failed,', tracing)
            raise Exception(ex)
    
    def load_df_intotable(self, df, schema_name, table_name, replace_append, tracing = False):
        """It loads the dataframe into a sql table. First it drops the table if exists and creates it again. Then loads the table with dataframe.
        
        Parameters:
            schemaname (str): schema name
            tbl_name (str): table name

        Returns: 
            Nothing
        """
        is_df_loaded = False
        try:
            trace.print(f'''load df into table, ''', tracing)
            if replace_append == 'r':
                do = 'replace'
            elif replace_append == 'a':
                do = 'append'
            else:
                do = 'append'

            with self.__engine.connect() as connection:
                df.to_sql(table_name
                          , connection
                          , schema = schema_name
                          , if_exists = do
                          , chunksize = 10000
                          , index = False)
            is_df_loaded = True
            trace.print(f'load df into table executed successfully : {schema_name}.{table_name}', tracing)
            return is_df_loaded
        except Exception as ex:
            trace.print(f'load df into table failed : {schema_name}.{table_name}', tracing)
            raise Exception(ex)
    



# ############ use/uncomment to pulich to pypi
import setuptools 
if __name__ == "__main__": 
   setuptools.setup()


# ############ use/uncomment for local installation
# import omshakya._bin_cc_or_sc.__version__ as v
# __VERSION__ = v.__VERSION__
# print(__VERSION__)

# from setuptools import setup, find_packages
# setup(name = 'omshakya',
#       version = '0.1.5',
#       packages = find_packages(),
#       description = '''A package created by om shakya, this provides several interfaces to sql
#       , mongodb, azure service bus, and other utility functions.
#       The author of the package call it "half done" and most of the re-usable code is modularize and
#       the developer can focus on the main business logics.''',
#       url = 'https://www.linkedin.com/in/om-shakya-49759120',
#       author = 'om shakya',
#       author_email = 'ommshakya@gmail.com',
#       license = 'MIT',
#       zip_safe = False
#       )


# ############# not working at all
# from setuptools import setup, find_packages
# setup(
#     name = 'omshakya',
#     version = '0.1',
#     packages = find_packages(),
#     install_requires = [
#         # list any dependencies your package has
#     ],
# )